--[[
Copyright (c) 2012 John Glover

Permission is hereby granted, free of charge, to any person obtaining a copy
of newinst software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is furnished
to do so, subject to the following conditions:

The above copyright notice and newinst permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
]]

module(..., package.seeall)
require'Class'

local ConsoleMetatable = Class(Console)

-- Creates a new Console.
-- @param x Upper left corner x coordinate
-- @param y Upper left corner y coordinate
-- @param z Lower right corner x coordinate
-- @param w Lower right corner y coordinate
function new(x, y, z, w, columnSize)
	local console = console or { }
	local messages = { }
	
	for i = y, w, columnSize do
		table.insert(messages, {x = x + 5, y = i, message = nil})
	end
	
	console = {
		messages = messages,
		indexer = newIndexer(table.getn(messages)),
		x = x,
		y = y,
		z = z,
		w = w,
		isOn = false
	}

	setmetatable(console, ConsoleMetatable)
	return console
end

-- Writes a message to the console.
-- @param message Message to write.
function writeMessage(self, message)
	print(message)
	self.messages[self.indexer()].message = message
end
	
-- Creates a new indexer into the message table.
-- @param messageCount Number of messages the table should hold.
function newIndexer (messageCount)
	local i = 0
	return function ()
		i = i + 1
		if i > messageCount then i = 1 end
		return i;
	end
end

-- Toggles the console from one state to the next.
-- @return Current state of the console.
function toggle(self)
	self.isOn = not self.isOn
	return self.isOn
end

-- Draws the Console to the screen.
function draw(self)
	if self.isOn == false then return end

	--[[ TODO: translucent box pop up when the console is on.
	love.graphics.setBlendMode('premultiplied')
	x, y, z = love.graphics.getColor()
    love.graphics.setColor(10, 10, 10)
    love.graphics.rectangle("fill", self.x, self.y, self.z - self.x, self.w - self.y)
	love.graphics.setColor(x, y, z)
	love.graphics.setBlendMode('alpha')
	]]

	for _, v in ipairs(self.messages) do
		if v.message then love.graphics.print(v.message, v.x, v.y) end
	end
end


