--[[
Copyright (c) 2012 John Glover

Permission is hereby granted, free of charge, to any person obtaining a copy
of newinst software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is furnished
to do so, subject to the following conditions:

The above copyright notice and newinst permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
]]

module(..., package.seeall)
require'Class'

local ShipMetatable = Class(Ship)

function new(acceleration, velocity, fuel, regeneration, cost)
	local ship = {
		acceleration = acceleration,
		maximumVelocity = velocity,
		maximumFuel = fuel,
		fuelRegeneration = regeneration,
		fuel = fuel,
		accelerationCost = cost,
		burnedOut = false,
		burnoutThreshold = 0.2,
	}

	setmetatable(ship, ShipMetatable)
	return ship
end

function getAcceleration(self)
	return self.acceleration
end

function getMaxVelocity(self)
	return self.maximumVelocity
end

function consumeFuel(self, dt, amount)
	if self.burnedOut then return false end

	local cost = (amount or self.accelerationCost) * dt

	if self.fuel - cost < 0 then 
		self.burnedOut = true
		return false 
	end
	
	self.fuel = self.fuel - cost
	return true
end

function regenerate(self, dt)
	self.fuel = math.min(self.fuel + self.fuelRegeneration * dt, self.maximumFuel)
	if self.burnedOut then
		if self.fuel / self.maximumFuel > self.burnoutThreshold then self.burnedOut = false end
	end
end

function queryFuel(self)
	return self.fuel / self.maximumFuel
end

function attachComponent(self, component)

end

function removeComponent(self)

end